#ifndef LINT
static char *rcsid="$Id: rulecompile.c,v 1.5 2002/09/05 13:36:21 crosser Exp $";
#endif

/*
	$Log: rulecompile.c,v $
	Revision 1.5  2002/09/05 13:36:21  crosser
	spell checking, more commentary, for release 1.00
	
	Revision 1.4  2002/08/26 16:02:59  crosser
	made different patterns for envelope, headers and body
	
	Revision 1.3  2002/08/25 19:05:34  crosser
	ready version 0.01
	
	Revision 1.2  2002/08/25 14:02:14  crosser
	implement special characters
	include README
	make test script
	
	Revision 1.1  2002/08/25 11:22:51  crosser
	work in progress
	
*/

/*
	Copyright (c) 2002 Eugene Crosser

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.

        rulecompile: command to generate precompiled file with
        reject patterns for use by contentfilter.  Try to replace
        the working file gracefully: create a new file in the same
        directory and then remove and replace the working file.
        This program also takes care of including "switchover" rules
        that distinguish between envelope, headers and body of the
        message.  This is done here to keep contentfilter as simple
        and small as possible.

        The program takes a single command line argument - source
        patterns definition filename (see spampatterns.sample for
        syntax).  Special name of '-' means read stdin.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifndef HAVE_MKSTEMP
# define mkstemp mktemp
#endif

#define CR '\r'
#define LF '\n'

static char *rulefile=RULEFILE;

int
main(int argc, char *argv[])
{
	char buf[256];
	char *tmpfile;
	FILE *sfp;
	int dfd;
	struct _rulechain {
		struct _rulechain *next;
		char txt[1];
	} *cur,*head[3],**curp[3];
	int count[3],total;
	size_t rulesize=3*sizeof(int);
	char *ruledata,*p,*q;
	int *rulecount;
	size_t *ruleoffset;
	int escaped;
	int lineno=0;
	int j;

	if (argc != 2) {
		fprintf(stderr,"usage: %s { filename | - }\n",argv[0]);
		return 1;
	}
	if (strcmp(argv[1],"-") == 0) sfp=stdin;
	else {
		if ((sfp=fopen(argv[1],"r")) == NULL) {
			perror(argv[1]);
			return 1;
		}
	}

	if ((tmpfile=(char*)malloc(strlen(rulefile)+8)) == NULL) {
		perror("malloc for tmp file name");
		return 1;
	}
	sprintf(tmpfile,"%s.XXXXXX",rulefile);
	if ((dfd=mkstemp(tmpfile)) == -1) {
		perror(tmpfile);
		return 1;
	}
#ifdef DEBUG
	fprintf(stderr,"temp file \"%s\"\n",tmpfile);
#endif

	total=0;

	count[0]=1;
	total++;
	head[0]=(struct _rulechain*)
			malloc(strlen("\nenv-end")+sizeof(struct _rulechain));
	head[0]->next=NULL;
	strcpy(head[0]->txt,"\nenv-end");
	rulesize+=strlen("\nenv-end")+1+sizeof(size_t*);
	curp[0]=&(head[0]->next);

	count[1]=1;
	total++;
	head[1]=(struct _rulechain*)
			malloc(strlen("\n\n")+sizeof(struct _rulechain));
	head[1]->next=NULL;
	strcpy(head[1]->txt,"\n\n");
	rulesize+=strlen("\nenv-end")+1+sizeof(size_t*);
	curp[1]=&(head[1]->next);

	count[2]=0;
	head[2]=NULL;
	curp[2]=&head[2];

	while (fgets(buf,sizeof(buf)-1,sfp)) {
		lineno++;
		if (buf[0] == '#') continue; /* skip comments */
		buf[sizeof(buf)-1]='\0';
		for (p=buf+strlen(buf)-1;p>buf && (*p==CR || *p==LF);p--)
			*p='\0';
		if (p <= buf) continue; /* skip empty lines */
		switch (buf[0]) {
		case 'E':
		case 'e':
			j=0;
			break;
		case 'H':
		case 'h':
			j=1;
			break;
		case 'B':
		case 'b':
			j=2;
			break;
		default:
			j=-1;
			break;
		}
		if (j < 0) {
			fprintf(stderr,"error in %s line %d:\n     \"%s\"\n",
				argv[1],lineno,buf);
			continue;
		}
		(*curp[j])=(struct _rulechain*)
			malloc(2*strlen(buf)+sizeof(struct _rulechain));
		(*curp[j])->next=NULL;
		escaped=0;
		for (p=buf+1,q=(*curp[j])->txt;*p;p++) {
			if (escaped) {
				escaped=0;
				*q++=*p;
			} else {
				escaped=0;
				switch (*p) {
				case '\\':
					escaped=1;
					break;
				case '^':
					*q++=LF;
					break;
				case '*':
					*q++=CR;
					break;
				default:
					*q++=*p;
					break;
				}
			}
		}
		*q='\0';
		rulesize+=strlen((*curp[j])->txt)+1+sizeof(size_t*);
		curp[j]=&((*curp[j])->next);
		count[j]++;
		total++;
	}
#ifdef DEBUG
	for (j=0;j<3;j++) {
		printf("section %d (%d entries):\n",j,count[j]);
		for (cur=head[j];cur;cur=cur->next) {
			printf("  line: \"%s\"\n",cur->txt);
		}
	}
	printf("total %d entries, %d bytes\n",total,rulesize);
#endif
	if ((ruledata=(char*)malloc(rulesize)) == NULL) {
		perror("malloc for ruledata");
		return 1;
	}
	rulecount=(int*)ruledata;
	ruleoffset=(size_t*)(rulecount+3);
	p=(char*)(ruleoffset+total);
	for (j=0;j<3;j++) {
		rulecount[j]=count[j];
		for (cur=head[j];cur;cur=cur->next) {
			*ruleoffset++=p-ruledata;
			strcpy(p,cur->txt);
			p+=strlen(cur->txt)+1;
		}
	}
	if (write(dfd,ruledata,rulesize) != rulesize) {
		perror("write tmpfile");
		return 1;
	}
	if (fchmod(dfd,0644) < 0) {
		perror("chmod a+w tmpfile");
		return 1;
	}
	if (close(dfd) < 0) {
		perror("close tmpfile");
		return 1;
	}
	if ((unlink(rulefile) < 0) && (errno != ENOENT)) {
		perror("unlink old rulefile");
		return 1;
	}
	if (link(tmpfile,rulefile) < 0) {
		perror("link new file to rulefile");
		return 1;
	}
	if (unlink(tmpfile) < 0) {
		perror("unlink tmpfile");
		return 1;
	}

	if (sfp != stdin) fclose(sfp);
	return 0;
}
